@@ -1,6 +1,6 @@
 ;;; bibtex.el --- BibTeX mode for GNU Emacs
 
-;; Copyright (C) 1992 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1994 Free Software Foundation, Inc.
 
 ;; Author: Bengt Martensson <ubrinf!mond!bengt>
 ;;	Mark Shapiro <shapiro@corto.inria.fr>
@@ -138,17 +138,17 @@
 ;;;   user visible variables added:
 ;;; 	 bibtex-maintain-sorted-entries
 ;;;   new local keybindings:
-;;; 	"	TeX-insert-quote
+;;; 	"	tex-insert-quote
 ;;; 	C-c$   ispell-bibtex-entry
 ;;; 	M-C-a  beginning-of-bibtex-entry
 ;;; 	M-C-e  end-of-bibtex-entry
 ;;; Mike Newton (newton@gumby.cs.caltech.edu) 90.11.17
 ;;;  * Handle items like
-;;;          title = poft # "Fifth Tri-quaterly" # random-conf,
+;;;          title = poft # "Fifth Tri-quarterly" # random-conf,
 ;;;    and   title = {This title is inside curlies}
 ;;;  * added user settable, always present, optional fields
 ;;;  * fixed 'bibtex-find-it's doc string's location
-;;;  * bibtex-field-text made more general (it wouldnt handle the # construct)
+;;;  * bibtex-field-text made more general (it wouldn't handle the # construct)
 ;;;		and it now handles a small subset of the {} cases
 
 ;;; Bengt Martensson, March 6
@@ -233,8 +233,8 @@
 ;;; these guys typically don't have autoloads...[alarson:19920131.1548CST]
 ;;; Check for fboundp first so that if user autoloads them from non standard 
 ;;; places, the users bindings will take precedence.
-(if (not (fboundp 'TeX-insert-quote))
-    (autoload 'TeX-insert-quote "tex-mode"))
+(if (not (fboundp 'tex-insert-quote))
+    (autoload 'tex-insert-quote "tex-mode"))
 (if (not (fboundp 'sort-subr))
     (autoload 'sort-subr "sort"))
 
@@ -308,11 +308,11 @@ string and the cdr the value to be inserted.")
     (define-key km "\C-c\C-d" 'bibtex-empty-field)
 
     ;; [alarson:19920131.1543CST]
-    (define-key km "\""   'TeX-insert-quote)
+    (define-key km "\""   'tex-insert-quote)
     (define-key km "\C-c$"   'ispell-bibtex-entry)
     (define-key km "\M-\C-a"   'beginning-of-bibtex-entry)
     (define-key km "\M-\C-e"   'end-of-bibtex-entry)
-    (define-key km "\C-ce"   'bibtex-entry)
+    (define-key km "\C-c\C-b"   'bibtex-entry)
 ;    (define-key km "\C-cn" 'narrow-to-bibtex-entry)
 
     (define-key km "\C-c\C-e\C-a" 'bibtex-Article)
@@ -455,8 +455,12 @@ types, e.g. a crossreference document of organization types.")
 ;;; Amendment:  I couldn't get a regexp with both "[]"'s and hyphen to
 ;;; work.  It looks like you need them both to be the first entries in a
 ;;; regexp pattern. [alarson:19930315.0900CST]
+;;;
+;;; New amendment[MB]: I have corrected the following regexp according to 
+;;; emacs19 rules.  I also added ']' to the list of characters allowed in 
+;;; a field name.
 
-(defconst bibtex-field-name "[A-Za-z][---A-Za-z0-9.:;?!`'()/*@_+=]*"
+(defconst bibtex-field-name "[A-Za-z][]A-Za-z0-9.:;?!`'()/*@_+=-]*"
   "Regexp defining the name part of a bibtex field.")
 
 ;; bibtex-field-text must be able to handle
@@ -469,7 +473,7 @@ types, e.g. a crossreference document of organization types.")
 ;; i have added a few of these, but not all! -- MON
 
 (defconst bibtex-field-const
-  "[0-9A-Za-z][---A-Za-z0-9:_+]*"
+  "[0-9A-Za-z][A-Za-z0-9:_+-]*"
   "Format of a bibtex field constant.")
 
 (defconst bibtex-field-string
@@ -491,7 +495,7 @@ an empty string, or a constant followed by one or more # / constant pairs.
 Also matches simple {...} patterns.")
 
 ;(defconst bibtex-field-text
-;  "\"[^\"]*[^\\\\]\"\\|\"\"\\|[0-9A-Za-z][---A-Za-z0-9:_+]*"
+;  "\"[^\"]*[^\\\\]\"\\|\"\"\\|[0-9A-Za-z][A-Za-z0-9:_+-]*"
 ;  "Regexp defining the text part of a bibtex field: either a string, or an empty string, or a constant.")
 
 (defconst bibtex-field
@@ -521,9 +525,16 @@ Also matches simple {...} patterns.")
 bibtex-reference-head")
 
 (defconst bibtex-reference
-  (concat bibtex-reference-head
-	  "\\([ \t\n]*" bibtex-field "\\)*"
-	  "[ \t\n]*[})]")
+  (concat
+   ;; This is the unparenthesized equivalent of bibtex-reference-head:
+   "^[ \t]*"
+   bibtex-reference-type
+   "[ \t]*[({]\\("
+   bibtex-field-name
+   "\\)"
+   ;; End of unparenthesized equivalent of bibtex-reference-head
+   "\\([ \t\n]*" bibtex-field "\\)*"
+   "[ \t\n]*[})]")
   "Regexp defining the format of a bibtex reference entry")
 (defconst bibtex-type-in-reference bibtex-type-in-head
   "The regexp subexpression number of the type part in bibtex-reference")
@@ -544,7 +555,7 @@ bibtex-reference")
 (defconst bibtex-text-in-string 2
   "The regexp subexpression of the text part in bibtex-string")
 
-(defconst bibtex-name-alignement 2
+(defconst bibtex-name-alignment 2
   "Alignment for the name part in BibTeX fields.
 Chosen on aesthetic grounds only.")
 
@@ -553,7 +564,7 @@ Chosen on aesthetic grounds only.")
 Equal to the space needed for the longest name part.")
 
 (defun bibtex-current-entry-label (&optional include-cite kill)
-  "Return the label of the bibtex entry containing, or preceeding point.
+  "Return the label of the bibtex entry containing, or preceding point.
 Optional argument INCLUDE-CITE, if true means put a '\\cite{}' around the
 returned value.  Second optional argument KILL, if true, means place the
 returned value in the kill buffer.  Interactively; providing prefix
@@ -563,7 +574,7 @@ Rationale:
 The intention is that someone will write a function that can be bound to
 a mouse key so that people entering TeX can just mouse on the bibtex entry
 and have the citation key inserted at the current point (which will almost
-certainly be in some other bufer).  In the interim this function is 
+certainly be in some other buffer).  In the interim this function is 
 marginally useful for keyboard binding and is not bound by default.  
 Suggested binding is ^C-k."
   (interactive (list current-prefix-arg t))
@@ -581,6 +592,7 @@ Suggested binding is ^C-k."
 
 ;;; bibtex mode:
 
+;;;###autoload
 (defun bibtex-mode () 
   "Major mode for editing bibtex files.
 
@@ -603,11 +615,6 @@ checks that no non-optional fields are empty.
 Use \\[bibtex-find-text] to position the dot at the end of the current field.
 Use \\[bibtex-next-field] to move to end of the next field.
 
-\\[bibtex-x-environment] binds a mode-specific X menu to control+right
-mouse button.
-\\[bibtex-sun-environment] binds a mode-specific Sun menu to right
-mouse button.
-
 The following may be of interest as well:
 
   Functions:
@@ -693,7 +700,7 @@ non-nil."
   (setq local-abbrev-table bibtex-mode-abbrev-table)
   (make-local-variable 'paragraph-start)
   (setq paragraph-start "^[ \f\n\t]*$")
-  (auto-fill-mode 1)			; nice alignements
+  (auto-fill-mode 1)			; nice alignments
   (setq left-margin (+ bibtex-text-alignment 1))
 
   (run-hooks 'bibtex-mode-hook))
@@ -801,7 +808,10 @@ With argument, show all text."
       (if arg
 	  (subst-char-in-region (point) (point-max) ?\r ?\n t)
 	(while (save-excursion (re-search-forward "\n[^@]" (point-max) t))
-	  (save-excursion (replace-regexp "\n\\([^@]\\)" "\r\\1"))))
+	  ;; (save-excursion (replace-regexp "\n\\([^@]\\)" "\r\\1"))
+	  (save-excursion
+	    (while (re-search-forward "\n\\([^@]\\)" nil t)
+	      (replace-match "\r\\1" nil nil)))))
       (setq selective-display (not arg))
       (set-buffer-modified-p modifiedp))))
 
@@ -817,7 +827,7 @@ If bibtex-sort-ignore-string-entries is true, @string entries will be ignored.
 
 Bugs:
   1. Text between the closing brace ending one bibtex entry, and the @ starting 
-     the next, is considered part of the PRECEEDING entry.  Perhaps it should be
+     the next, is considered part of the PRECEDING entry.  Perhaps it should be
      part of the following entry."
   (interactive)
   (save-restriction
@@ -829,7 +839,7 @@ Bugs:
 			(end-of-bibtex-entry)
 			(point)))
     (sort-subr nil			; reversep
-	       ;; begining of record function
+	       ;; beginning of record function
 	       'forward-line
 	       ;; end of record function
 	       (function (lambda () (and (re-search-forward "}\\s-*\n[\n \t]*@" nil 'move)
@@ -1024,7 +1034,7 @@ as such should normally be set via a file local variable entry.")
   (let ((name  (if (consp e-t) (car e-t) e-t))
 	(value (if (consp e-t) (cdr e-t) "")))
     (insert ",\n")
-    (indent-to-column bibtex-name-alignement)
+    (indent-to-column bibtex-name-alignment)
     (insert name " = ")
     (indent-to-column bibtex-text-alignment)
     ;; lucid emacs prin1-to-string breaks the undo chain.  When they fix
@@ -1486,57 +1496,69 @@ an undefined location.
     (skip-whitespace-and-comments)))
 
 
+;;; Menus for bibtex mode
 
-;;; X window menus for bibtex mode
+(define-key bibtex-mode-map [menu-bar entry-types]
+  (cons "Entry Types" (make-sparse-keymap "Entry Types")))
 
-(defun bibtex-x-help (arg)
-  "Mouse commands for BibTeX mode"
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-InProceedings]
+  '(" article in conference Proceedings " . bibtex-InProceedings))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Article]
+  '("        Article in journal         " . bibtex-Article))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Book]
+  '("               Book                " . bibtex-Book))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Booklet]
+  '("             Booklet               " . bibtex-Booklet))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-InProceedings]
+  '("            Conference             " . bibtex-InProceedings))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-MastersThesis]
+  '("         Master's Thesis           " . bibtex-MastersThesis))
+;define-key bibtex-mode-map [menu-bar entry-types bibtex-DEAthesis]
+;'(("            DEA Thesis             " . bibtex-DEAthesis))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-PhdThesis]
+  '("            Phd. Thesis            " . bibtex-PhdThesis))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-TechReport]
+  '("         Technical Report          " . bibtex-TechReport))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Manual]
+  '("         technical Manual          " . bibtex-Manual))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Proceedings]
+  '("      conference Proceedings       " . bibtex-Proceedings))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-InBook]
+  '("        a chapter in a Book        " . bibtex-InBook))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-InCollection]
+  '("    an article in a Collection     " . bibtex-InCollection))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Misc]
+  '("           miscellaneous           " . bibtex-Misc))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-Unpublished]
+  '("            unpublished            " . bibtex-Unpublished))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-string]
+  '("              string               " . bibtex-string))
+(define-key bibtex-mode-map [menu-bar entry-types bibtex-preamble]
+  '("             preamble              " . bibtex-preamble))
 
-  (let ((selection
-	 (x-popup-menu
-	  arg
-	  '("BibTeX commands"
-	    ("BibTeX entry types"
-	     (" article in conference Proceedings " . bibtex-InProceedings)
-	     ("        Article in journal         " . bibtex-Article)
-	     ("               Book                " . bibtex-Book)
-	     ("             Booklet               " . bibtex-Booklet)
-	     ("            Conference             " . bibtex-InProceedings)
-	     ("         Master's Thesis           " . bibtex-MastersThesis)
-;	     ("            DEA Thesis             " . bibtex-DEAthesis)
-	     ("            Phd. Thesis            " . bibtex-PhdThesis)
-	     ("         Technical Report          " . bibtex-TechReport)
-	     ("         technical Manual          " . bibtex-Manual)
-	     ("      conference Proceedings       " . bibtex-Proceedings)
-	     ("        a chapter in a Book        " . bibtex-InBook)
-	     ("    an article in a Collection     " . bibtex-InCollection)
-	     ("           miscellaneous           " . bibtex-Misc)
-	     ("            unpublished            " . bibtex-Unpublished)
-	     ("              string               " . bibtex-string)
-	     ("             preamble              " . bibtex-preamble)
-	     )
-	    ("Moving around and editing"
-	     ("            next field             " . bibtex-next-field)
-	     ("          to end of field          " . bibtex-find-text)
-	     ("snatch from similar preceding field" . bibtex-pop-previous)
-	     ("snatch from similar following field" . bibtex-pop-next)
-	     ("            remove OPT             " . bibtex-remove-OPT)
-	     ("           remove quotes           "
-	      . bibtex-remove-double-quotes)
-	     ("          clean up entry           " . bibtex-clean-entry)
-	     )
-	    ("help"
-	     ("       describe BibTeX mode        " . describe-mode)
-	     )))))
-    (and selection (call-interactively selection))))
-
-(defun bibtex-x-environment ()
-  "Set up X menus for BibTeX mode.  Call it as bibtex-mode-hook, or interactively"
-  (interactive)
-  (require 'x-mouse)
-  (define-key mouse-map x-button-c-right 'bibtex-x-help)
-  )
+(define-key bibtex-mode-map [menu-bar move/edit]
+  (cons "Bibtex Edit" (make-sparse-keymap "Bibtex Edit")))
 
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-next-field]
+  '("            next field             " . bibtex-next-field))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-find-text]
+  '("          to end of field          " . bibtex-find-text))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-pop-previous]
+  '("snatch from similar preceding field" . bibtex-pop-previous))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-pop-next]
+  '("snatch from similar following field" . bibtex-pop-next))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-remove-OPT]
+  '("            remove OPT             " . bibtex-remove-OPT))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-remove-double-quotes]
+  '("           remove quotes           " . bibtex-remove-double-quotes))
+(define-key bibtex-mode-map [menu-bar move/edit bibtex-clean-entry]
+  '("          clean up entry           " . bibtex-clean-entry))
+(define-key bibtex-mode-map [menu-bar move/edit find-bibtex-duplicates]
+  '("          find duplicates          " . find-bibtex-duplicates))
+(define-key bibtex-mode-map [menu-bar move/edit sort-bibtex-entries]
+  '("           sort entries            " . sort-bibtex-entries))
+(define-key bibtex-mode-map [menu-bar move/edit validate-bibtex-buffer]
+  '("         validate entries          " . validate-bibtex-buffer))
 
 
 ;; Please don't send anything to bug-gnu-emacs about these Sunwindows functions
