@@ -3,8 +3,8 @@
 ;; Author: Sebastian Kremer <sk@thp.uni-koeln.de>
 ;;	Lawrence R. Dodd <dodd@roebling.poly.edu>
 ;; Maintainer: Lawrence R. Dodd <dodd@roebling.poly.edu>
-;; Version: 2.37+
-;; Date: 1994/08/18 19:27:42
+;; Version: 2.27
+;; Date: 1994/04/05 12:45:30
 ;; Keywords: dired extensions
 
 ;; Copyright (C) 1993, 1994 Free Software Foundation
@@ -22,116 +22,109 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; Commentary:
 
-;; This is Sebastian Kremer's excellent dired-x.el (Dired Extra), version
-;; 1.191, hacked up for GNU Emacs 19.  Redundant or conflicting material
-;; has been removed or renamed in order to work properly with dired of
-;; GNU Emacs 19.  All suggestions or comments are most welcomed.
+;;; This is Sebastian Kremer's excellent dired-x.el (Dired Extra), version
+;;; 1.191, hacked up for GNU Emacs 19.  Redundant or conflicting material
+;;; has been removed or renamed in order to work properly with dired of
+;;; GNU Emacs 19.  All suggestions or comments are most welcomed.
+;;;  
+;;; *Please* see the info pages.
 
-;;  
-;; Please, PLEASE, *PLEASE* see the info pages.
-;; 
+;;; BUGS: Type M-x dired-x-submit-report and a report will be generated.
 
-;; BUGS: Type M-x dired-x-submit-report and a report will be generated.
+;;; INSTALLATION: In your ~/.emacs,
+;;;
+;;; (add-hook 'dired-load-hook
+;;;           (function (lambda ()
+;;;                       (load "dired-x")
+;;;                       ;; Set variables here.  For example:
+;;;                       ;; (setq dired-guess-shell-gnutar "gtar")
+;;;                       ;; (setq dired-omit-files-p t)
+;;;                       )))
+;;;
+;;; At load time dired-x.el will install itself, redefine some functions, and
+;;; bind some dired keys.  *Please* see the info pages for more details.
 
-;; INSTALLATION: In your ~/.emacs,
-;;
-;; (add-hook 'dired-load-hook
-;;           (function (lambda ()
-;;                       (load "dired-x")
-;;                       ;; Set global variables here.  For example:
-;;                       ;; (setq dired-guess-shell-gnutar "gtar")
-;;                       )))
-;; (add-hook 'dired-mode-hook
-;;           (function (lambda ()
-;;                       ;; Set buffer-local variables here.  For example:
-;;                       ;; (setq dired-omit-files-p t)
-;;                       )))
-;;
-;; At load time dired-x.el will install itself, redefine some functions, and
-;; bind some dired keys.  *Please* see the info pages for more details.
+;;; User defined variables:
+;;;
+;;;      dired-bind-vm
+;;;      dired-vm-read-only-folders
+;;;      dired-bind-jump
+;;;      dired-bind-info
+;;;      dired-bind-man
+;;;      dired-find-subdir
+;;;      dired-enable-local-variables
+;;;      dired-local-variables-file
+;;;      dired-guess-shell-gnutar
+;;;      dired-guess-shell-gzip-quiet
+;;;      dired-guess-shell-znew-switches
+;;;      dired-guess-shell-alist-user
+;;;      dired-clean-up-buffers-too
+;;;      dired-omit-files-p
+;;;      dired-omit-files
+;;;      dired-omit-extensions
+;;;
+;;; To find out more about these variables, load this file, put your cursor at
+;;; the end of any of the variable names, and hit C-h v [RET].  *Please* see
+;;; the info pages for more details.
 
-;; CAUTION: If you are using a version of GNU Emacs earlier than 19.20 than
-;; you may have to edit dired.el.  The copy of dired.el in GNU Emacs versions
-;; earlier than 19.20 incorrectly had the call to run-hooks *before* the call
-;; to provide.  In such a case, it is possible that byte-compiling and/or
-;; loading dired can cause an infinite loop.  To prevent this, make sure the
-;; line of code
-;;  
-;;         (run-hooks 'dired-load-hook) 
-;;  
-;; is the *last* executable line in the file dired.el.  That is, make sure it
-;; comes *after* the line
-;;  
-;;         (provide 'dired) 
-;; 
-;; *Please* see the info pages for more details. 
-
-;; User defined variables:
-;;
-;;      dired-bind-vm
-;;      dired-vm-read-only-folders
-;;      dired-bind-jump
-;;      dired-bind-info
-;;      dired-bind-man
-;;      dired-x-hands-off-my-keys 
-;;      dired-find-subdir
-;;      dired-enable-local-variables
-;;      dired-local-variables-file
-;;      dired-guess-shell-gnutar
-;;      dired-guess-shell-gzip-quiet
-;;      dired-guess-shell-znew-switches
-;;      dired-guess-shell-alist-user
-;;      dired-clean-up-buffers-too
-;;      dired-omit-files-p
-;;      dired-omit-files
-;;      dired-omit-extensions
-;;
-;; To find out more about these variables, load this file, put your cursor at
-;; the end of any of the variable names, and hit C-h v [RET].  *Please* see
-;; the info pages for more details.
-
-;; When loaded this code redefines the following functions of GNU Emacs
-;;
-;;   Function                         Found in this file of GNU Emacs
-;;   --------                         -------------------------------
-;;   dired-clean-up-after-deletion    ../lisp/dired.el
-;;   dired-find-buffer-nocreate       ../lisp/dired.el
-;;   dired-initial-position           ../lisp/dired.el
-;;
-;;   dired-add-entry                  ../lisp/dired-aux.el
-;;   dired-read-shell-command         ../lisp/dired-aux.el
-;;
-;; One drawback is that dired-x.el will load dired-aux.el as soon as dired is
-;; loaded.  Thus, the advantage of separating out non-essential dired stuff
-;; into dired-aux.el and only loading when necessary will be lost.  Please
-;; note also that some of the comments in dired.el and dired-aux.el are
-;; Kremer's that referred to the old dired-x.el.  This now should be referring
-;; to this program.  (This is also a good reason to call this dired-x.el
-;; instead of dired-x19.el.)
+;;; When loaded this code redefines the following functions of GNU Emacs
+;;;
+;;;   Function                         Found in this file of GNU Emacs
+;;;   --------                         -------------------------------
+;;;   dired-clean-up-after-deletion    ../lisp/dired.el
+;;;   dired-find-buffer-nocreate       ../lisp/dired.el
+;;;   dired-initial-position           ../lisp/dired.el
+;;;   dired-up-directory               ../lisp/dired.el
+;;;
+;;;   dired-add-entry                  ../lisp/dired-aux.el
+;;;   dired-read-shell-command         ../lisp/dired-aux.el
+;;;
+;;; One drawback is that dired-x.el will load dired-aux.el as soon as dired is
+;;; loaded.  Thus, the advantage of separating out non-essential dired stuff
+;;; into dired-aux.el and only loading when necessary will be lost.  Please
+;;; note also that some of the comments in dired.el and dired-aux.el are
+;;; Kremer's that referred to the old dired-x.el.  This now should be referring
+;;; to this program.  (This is also a good reason to call this dired-x.el
+;;; instead of dired-x19.el.)
 
 
-;;; Code:
+;;;; Code:
 
-;; LOAD.
+;;; LOAD.
 
-;; This is a no-op if dired-x is being loaded via `dired-load-hook'.  It is
-;; here in case the user has autoloaded dired-x via the dired-jump key binding
-;; (instead of autoloading to dired as is suggested in the info-pages).
+;;; This is a no-op if dired-x is being loaded via `dired-load-hook'.  It is
+;;; here in case the user has autoloaded dired-x via the dired-jump key binding
+;;; (instead of autoloading to dired as is suggested in the info-pages).
 
-(require 'dired)
+;;; WARNING: The copy of dired.el in GNU Emacs versions earlier than 19.20 had
+;;; the `provide' *after* the `run-hooks'.  In such a case, loading dired below
+;;; will cause an infinite loop.  To prevent this we test the value of the GNU
+;;; Emacs major version number before requiring dired.
 
-;; We will redefine some functions and also need some macros so we need to
-;; load dired stuff of GNU Emacs.
+(if (string< "19.19"
+             ;; Compare with major version number (i.e., 19.22 not 19.22.11).
+             (substring emacs-version 0
+                        (and (string-match "^[0-9]*\\.[0-9]*" emacs-version)
+                             (match-end 0))))
+    (require 'dired))
 
-(require 'dired-aux)
+;;; We will redefine some functions and also need some macros so we need to
+;;; load dired stuff of GNU Emacs.  Since dired-aux.el (at least up to GNU
+;;; Emacs 19.22) does not `provide' itself, we do it here.  This avoids the
+;;; possibility recursive loading because of the nasty `eval-when-compile' that
+;;; is in dired-aux.el.
 
-;;; User-defined variables.
+(and (not (featurep 'dired-aux))
+     (load "dired-aux" nil t)
+     (not (featurep 'dired-aux))
+     (provide 'dired-aux))
+
+;;;; User-defined variables.
 
 (defvar dired-bind-vm nil
   "*t says \"V\" in dired-mode will `dired-vm', otherwise \"V\" is `dired-rmail'.
@@ -159,19 +152,15 @@ Read-only folders only work in VM 5, not in VM 4.")
 Use \\[dired-omit-toggle] to toggle its value.
 Uninteresting files are those whose filenames match regexp `dired-omit-files',
 plus those ending with extensions in `dired-omit-extensions'.")
-(make-variable-buffer-local 'dired-omit-files-p)
 
-(defvar dired-omit-files "^#\\|^\\.$\\|^\\.\\.$"
-  "*Filenames matching this regexp will not be displayed.
-This only has effect when `dired-omit-files-p' is t.  See interactive function
-`dired-omit-toggle' \(\\[dired-omit-toggle]\) and variable
-`dired-omit-extensions'.  The default is to omit  `.', `..', and auto-save
-files.")
+(defvar dired-omit-files "^#\\|\\.$"
+  "*Filenames matching this regexp will not be displayed (buffer-local).
+This only has effect when `dired-omit-files-p' is t.
+See also `dired-omit-extensions'.")
 
 (defvar dired-find-subdir nil           ; t is pretty near to DWIM...
-  "*If non-nil, Dired always finds a directory in a buffer of its own.
-If nil, Dired finds the directory as a subdirectory in some other buffer
-if it is present as one.
+  "*If non-nil, Dired does not make a new buffer for a directory if it
+can be found (perhaps as subdir) in some existing Dired buffer.
 
 If there are several Dired buffers for a directory, the most recently
 used is chosen.
@@ -203,7 +192,7 @@ to nil: a pipe using `zcat' or `gunzip -c' will be used.")
 (defvar dired-clean-up-buffers-too t
   "*t says offer to kill buffers visiting files and dirs deleted in dired.")
 
-;;; KEY BINDINGS.
+;;;; KEY BINDINGS.
 
 (define-key dired-mode-map "\M-o" 'dired-omit-toggle)
 (define-key dired-mode-map "\M-(" 'dired-mark-sexp)
@@ -230,7 +219,7 @@ to nil: a pipe using `zcat' or `gunzip -c' will be used.")
       (define-key global-map "\C-x4\C-j" 'dired-jump-other-window)))
 
 
-;;; Install into appropriate hooks.
+;;;; Install into appropriate hooks.
 
 (add-hook 'dired-mode-hook 'dired-extra-startup)
 (add-hook 'dired-after-readin-hook 'dired-omit-expunge)
@@ -286,9 +275,9 @@ See also functions
   (dired-omit-startup))
 
 
-;;; BUFFER CLEANING.
+;;;; BUFFER CLEANING.
 
-;; REDEFINE.
+;;; REDEFINE.
 (defun dired-clean-up-after-deletion (fn)
 
   ;; Clean up after a deleted file or directory FN.
@@ -307,7 +296,7 @@ See also functions
                                 (file-name-nondirectory fn)))
                (save-excursion ; you never know where kill-buffer leaves you
                  (kill-buffer buf))))
-        (let ((buf-list (dired-buffers-for-dir (expand-file-name fn)))
+        (let ((buf-list (dired-buffers-for-dir fn))
               (buf nil))
           (and buf-list
                (y-or-n-p (format "Kill dired buffer%s of %s, too? "
@@ -320,7 +309,7 @@ See also functions
   )
 
 
-;;; EXTENSION MARKING FUNCTIONS.
+;;;; EXTENSION MARKING FUNCTIONS.
 
 ;;; Mark files with some extension.
 (defun dired-mark-extension (extension &optional marker-char)
@@ -396,9 +385,8 @@ See variables `dired-texinfo-unclean-extensions',
                                 dired-tex-unclean-extensions
                                 (list ".dvi"))))
 
-;;; JUMP.
+;;;; JUMP.
 
-;;;###autoload
 (defun dired-jump (&optional other-window)
   "Jump to dired buffer corresponding to current buffer.
 If in a file, dired the current directory and move to file's line.
@@ -435,8 +423,30 @@ buffer and try again."
   "Like \\[dired-jump] (dired-jump) but in other window."
   (interactive)
   (dired-jump t))
+
+;;; REDEFINE.
+;;; This replaces the version in dired.el
+;;; It simply adds the OTHER-WINDOW option to the one in dired.el.
+(defun dired-up-directory (&optional other-window)
+  "Run dired on parent directory of current directory.
+Find the parent directory either in this buffer or another buffer.
+Finds in current window or in other window with optional OTHER-WINDOW.
+Creates a buffer if necessary."
+  (interactive "P")
+  (let* ((dir (dired-current-directory))
+         (up (file-name-directory (directory-file-name dir))))
+    (or (dired-goto-file (directory-file-name dir))
+        ;; Only try dired-goto-subdir if buffer has more than one dir.
+        (and (cdr dired-subdir-alist)
+             (dired-goto-subdir up))
+        (progn
+          (if other-window
+              (dired-other-window up)
+            (dired up))
+          (dired-goto-file dir)))))
+
 
-;;; TOGGLE.
+;;;; TOGGLE.
 ;;; Toggle marked files with unmarked files.
 
 (defun dired-do-toggle ()
@@ -463,7 +473,7 @@ As always, hidden subdirs are not affected."
         (forward-line 1)))))
 
 
-;;; COPY NAMES OF MARKED FILES INTO KILL-RING.
+;;;; COPY NAMES OF MARKED FILES INTO KILL-RING.
 
 (defun dired-copy-filename-as-kill (&optional arg)
   "Copy names of marked (or next ARG) files into the kill ring.
@@ -492,7 +502,7 @@ You can then feed the file name(s) to other commands with \\[yank]."
     (message "%s" string)))
 
 
-;;; OMITTING.
+;;;; OMITTING.
 
 ;;; Enhanced omitting of lines from directory listings.
 ;;; Marked files are never omitted.
@@ -511,12 +521,13 @@ whole pathname.")
 Should never be used as marker by the user or other packages.")
 
 (defun dired-omit-startup ()
+  (make-local-variable 'dired-omit-files-p)
   (or (assq 'dired-omit-files-p minor-mode-alist)
       (setq minor-mode-alist
             (append '((dired-omit-files-p " Omit")) minor-mode-alist))))
 
 (defun dired-omit-toggle (&optional flag)
-  "Toggle omitting files matching `dired-omit-files' and `dired-omit-extensions'.
+  "Toggle between displaying and omitting files matching `dired-omit-files'.
 With an arg, and if omitting was off, don't toggle and just mark the
   files but don't actually omit them.
 With an arg, and if omitting was on, turn it off but don't refresh the buffer."
@@ -538,13 +549,10 @@ With an arg, and if omitting was on, turn it off but don't refresh the buffer."
           dired-latex-unclean-extensions
           dired-bibtex-unclean-extensions
           dired-texinfo-unclean-extensions)
-  "If non-nil, a list of extensions \(strings\) to omit from Dired listings.  
-Defaults to elements of `completion-ignored-extensions',
-`dired-latex-unclean-extensions', `dired-bibtex-unclean-extensions', and
-`dired-texinfo-unclean-extensions'.  
-
-See interactive function `dired-omit-toggle' \(\\[dired-omit-toggle]\) and
-variables `dired-omit-files-p' and `dired-omit-files'.")
+  "If non-nil, a list of extensions (strings) to omit from Dired
+listings.  Defaults to the elements of
+`completion-ignored-extensions', `dired-latex-unclean-extensions',
+`dired-bibtex-unclean-extensions' and `dired-texinfo-unclean-extensions'.")
 
 (defun dired-omit-expunge (&optional regexp)
   "Erases all unmarked files matching REGEXP.
@@ -558,7 +566,6 @@ This functions works by temporarily binding `dired-marker-char' to
   (interactive "sOmit files (regexp): ")
   (if dired-omit-files-p
       (let ((omit-re (or regexp (dired-omit-regexp)))
-            (old-modified-p (buffer-modified-p))
             count)
         (or (string= omit-re "")
             (let ((dired-marker-char dired-omit-marker-char))
@@ -566,14 +573,9 @@ This functions works by temporarily binding `dired-marker-char' to
               (if (dired-mark-unmarked-files omit-re nil nil dired-omit-localp)
                   (progn
                     (setq count (dired-do-kill-lines nil "Omitted %d line%s."))
-                    (force-mode-line-update))
+                    ;; Force an update of modeline.
+                    (set-buffer-modified-p (buffer-modified-p)))
                 (message "(Nothing to omit)"))))
-        ;; Try to preserve modified state of buffer.  So `%*' doesn't appear
-        ;; in mode-line of omitted buffers.
-        (set-buffer-modified-p (and old-modified-p 
-                                    (save-excursion
-                                      (goto-char (point-min))
-                                      (re-search-forward dired-re-mark nil t))))
         count)))
 
 (defun dired-omit-regexp ()
@@ -640,7 +642,7 @@ Second optional argument LOCALP is as in `dired-get-filename'."
 (fset 'dired-add-entry 'dired-omit-new-add-entry)
 
 
-;;; VIRTUAL DIRED MODE.
+;;;; VIRTUAL DIRED MODE.
 
 ;;; For browsing `ls -lR' listings in a dired-like fashion.
 
@@ -759,13 +761,13 @@ to put saved dired buffers automatically into virtual dired mode.
 
 Also useful for `auto-mode-alist' (which see) like this:
 
-  \(setq auto-mode-alist (cons '(\"[^/]\\.dired\\'\" . dired-virtual-mode)
+  \(setq auto-mode-alist (cons '(\"[^/]\\.dired$\" . dired-virtual-mode)
                               auto-mode-alist)\)"
   (interactive)
   (dired-virtual (dired-virtual-guess-dir)))
 
 
-;;; SMART SHELL.
+;;;; SMART SHELL.
 
 ;;; An Emacs buffer can have but one working directory, stored in the
 ;;; buffer-local variable `default-directory'.  A Dired buffer may have
@@ -799,7 +801,7 @@ cases in variable `default-directory-alist' (which see)."
     (shell-command cmd insert)))
 
 
-;;; LOCAL VARIABLES FOR DIRED BUFFERS.
+;;;; LOCAL VARIABLES FOR DIRED BUFFERS.
 
 ;;; Brief Description:
 ;;;
@@ -820,7 +822,7 @@ cases in variable `default-directory-alist' (which see)."
 ;;;   `dired-enable-local-variables' and run `hack-local-variables' on the
 ;;;   Dired Buffer.
 
-(defvar dired-local-variables-file (convert-standard-filename ".dired")
+(defvar dired-local-variables-file ".dired"
   "Filename, as string, containing local dired buffer variables to be hacked.
 If this file found in current directory, then it will be inserted into dired
 buffer and `hack-local-variables' will be run.  See Emacs Info pages for more
@@ -870,7 +872,7 @@ dired."
     (dired-revert)))
 
 
-;;; GUESS SHELL COMMAND.
+;;;; GUESS SHELL COMMAND.
 
 ;;; Brief Description:
 ;;;
@@ -887,11 +889,11 @@ dired."
 ;;;   that matches the first file in the file list.
 ;;;
 ;;; * If the REGEXP matches all the entries of the file list then evaluate
-;;;   COMMAND, which is either a string or a Lisp expression returning a
+;;;   COMMAND, which is either a string or an elisp expression returning a
 ;;;   string.  COMMAND may be a list of commands.
 ;;;
 ;;; * Return this command to `dired-guess-shell-command' which prompts user
-;;;   with it.  The list of commands is temporarily put into the history list.
+;;;   with it.  The list of commands are temporaily put into the history list.
 ;;;   If a command is used successfully then it is stored permanently in
 ;;;   `dired-shell-command-history'.
 
@@ -1114,7 +1116,7 @@ You can set this variable in your ~/.emacs.  For example, to add rules for
    files))                                       ; FILES
 
 
-;;; RELATIVE SYMBOLIC LINKS.
+;;;; RELATIVE SYMBOLIC LINKS.
 
 (defvar dired-keep-marker-relsymlink ?S
   "See variable `dired-keep-marker-move'.")
@@ -1145,7 +1147,7 @@ results in
                   ;; `/', so NEXT is *one plus* the result of the
                   ;; string-match.
                   ;; E.g., consider the case of linking "/tmp/a/abc"
-                  ;; to "/tmp/abc" erroneously giving "/tmp/a" instead
+                  ;; to "/tmp/abc" erronously giving "/tmp/a" instead
                   ;; of "/tmp/" as common initial component
                   (string-equal (substring file1 0 next)
                                 (substring file2 0 next)))
@@ -1197,7 +1199,7 @@ for more info."
    "RelSymLink" nil regexp newname whole-path dired-keep-marker-relsymlink))
 
 
-;;; VISIT ALL MARKED FILES SIMULTANEOUSLY.
+;;;; VISIT ALL MARKED FILES SIMULTANEOUSLY.
 
 ;;; Brief Description:
 ;;;
@@ -1279,18 +1281,58 @@ To display just marked files, type \\[delete-other-windows] first."
       (setq file-list (cdr file-list)))))
 
 
-;;; MISCELLANEOUS COMMANDS.
+;;;; MISCELLANEOUS COMMANDS.
 
 ;;; Run man on files.
 
 (defun dired-man ()
   "Run man on this file.  Display old buffer if buffer name matches filename.
-Uses ../lisp/man.el of \\[manual-entry] fame."
+Results displayed based on value of `Man-notify'.  See that variable."
   (interactive)
-  (require 'man)
-  (let ((file (dired-get-filename))
-        (manual-program "nroff -man -h"))
-    (Man-getpage-in-background file)))
+  (let* ((file (dired-get-filename))
+         (string (format "*man %s*" (file-name-nondirectory file)))
+         (Man-buffer (get-buffer string))
+         (msg "Expanding manual page...cleaning...done"))
+
+    ;; If Man-buffer already exists and has not been modified, display it.
+    ;; Otherwise, create a fresh one.
+    (if (and Man-buffer
+             (save-excursion
+               (set-buffer Man-buffer)
+               (not (buffer-modified-p))
+               buffer-read-only))
+
+        (setq msg "Displaying pre-existing manual page.")
+
+      ;; Create Man-buffer.
+      (save-excursion
+
+        ;; Prepare buffer.
+        (setq Man-buffer (get-buffer-create string))
+        (set-buffer Man-buffer)
+        (setq buffer-read-only nil)
+        (erase-buffer)
+
+        ;; Expand and clean man page.
+        (message "Expanding manual page...")
+        (call-process shell-file-name nil t nil "-c"
+                      (concat " nroff -man -h " file))
+        (message "Expanding manual page...cleaning...")
+        (call-process-region (point-min) (point-max)
+                             shell-file-name t t nil "-c" " col -b")
+        (goto-char (point-min))
+
+        ;; Reset buffer.
+        (setq buffer-read-only t)
+        (buffer-disable-undo (current-buffer))
+        (set-buffer-modified-p nil)))
+
+    ;; Display results.  Use display function of ../lisp/man.el whose behavior
+    ;; is determined by user-defined variable Man-notify.
+    (require 'man)
+    (Man-notify-when-ready Man-buffer)
+    ;; Overrides any message issued by above function.
+    (message msg)))
 
 ;;; Run Info on files.
 
@@ -1335,7 +1377,7 @@ See also variable `dired-vm-read-only-folders'."
     (dired-rmail)))
 
 
-;;; MISCELLANEOUS INTERNAL FUNCTIONS.
+;;;; MISCELLANEOUS INTERNAL FUNCTIONS.
 
 (or (fboundp 'dired-old-find-buffer-nocreate)
     (fset 'dired-old-find-buffer-nocreate
@@ -1343,22 +1385,19 @@ See also variable `dired-vm-read-only-folders'."
 
 ;;; REDEFINE.
 ;;; Redefines dired.el's version of `dired-find-buffer-nocreate'
-(defun dired-find-buffer-nocreate (dirname &optional mode)
-  (if (and dired-find-subdir
-	   ;; don't try to find a wildcard as a subdirectory
-	   (string-equal dirname (file-name-directory dirname)))
+(defun dired-find-buffer-nocreate (dirname)
+  (if dired-find-subdir
       (let* ((cur-buf (current-buffer))
-	     (buffers (nreverse
-		       (dired-buffers-for-dir (expand-file-name dirname))))
-	     (cur-buf-matches (and (memq cur-buf buffers)
-				   ;; wildcards must match, too:
-				   (equal dired-directory dirname))))
-	;; We don't want to switch to the same buffer---
-	(setq buffers (delq cur-buf buffers));;need setq with delq
-	(or (car (sort buffers (function dired-buffer-more-recently-used-p)))
-	    ;; ---unless it's the only possibility:
-	    (and cur-buf-matches cur-buf)))
-    (dired-old-find-buffer-nocreate dirname mode)))
+             (buffers (nreverse (dired-buffers-for-dir dirname)))
+             (cur-buf-matches (and (memq cur-buf buffers)
+                                   ;; wildcards must match, too:
+                                   (equal dired-directory dirname))))
+        ;; We don't want to switch to the same buffer---
+        (setq buffers (delq cur-buf buffers));;need setq with delq
+        (or (car (sort buffers (function dired-buffer-more-recently-used-p)))
+            ;; ---unless it's the only possibility:
+            (and cur-buf-matches cur-buf)))
+    (dired-old-find-buffer-nocreate dirname)))
 
 ;; This should be a builtin
 (defun dired-buffer-more-recently-used-p (buffer1 buffer2)
@@ -1415,7 +1454,7 @@ PREDICATE is a lisp expression that can refer to the following symbols:
 
     inode  [integer] the inode of the file (only for ls -i output)
     s      [integer] the size of the file for ls -s output
-                     (usually in blocks or, with -k, in KByte)
+                     (ususally in blocks or, with -k, in KByte)
     mode   [string]  file permission bits, e.g. \"-rw-r--r--\"
     nlink  [integer] number of links to file
     uid    [string]  owner
@@ -1498,72 +1537,25 @@ to mark all zero length files."
      (format "'%s file" predicate))))
 
 
-;;; FIND FILE AT POINT.
+;;;; FIND FILE AT POINT.
+(defun dired-find-this-file (&optional other-window)
+  "Edit filename or directory at point.
+Switch to a buffer visiting filename, creating one if none already exists.
+With non-nil prefix argument OTHER-WINDOW do so in the other window.
 
-(defvar dired-x-hands-off-my-keys t
-  "*t means don't bind `dired-x-find-file' over `find-file' on keyboard.
-Similarly for `dired-x-find-file-other-window' over `find-file-other-window'.
-If you change this variable after dired-x.el is loaded then do
-\\[dired-x-bind-find-file].")
+Useful for editing the file mentioned in the buffer you are viewing, or to
+test if that file exists.  Use minibuffer after snatching the filename."
 
-;;; Bind `dired-x-find-file{-other-window}' over wherever
-;;; `find-file{-other-window}' is bound?
-(defun dired-x-bind-find-file ()
-  "Bind `dired-x-find-file' in place of `find-file' \(or reverse\).
-Similarly for `dired-x-find-file-other-window' and `find-file-other-window'.
-Binding direction based on `dired-x-hands-off-my-keys'.
-This function part of `after-init-hook'."
-  (interactive)
-  (if (interactive-p)
-      (setq dired-x-hands-off-my-keys
-            (not (y-or-n-p "Bind dired-x-find-file over find-file? "))))
-  (cond ((not dired-x-hands-off-my-keys)
-         (substitute-key-definition 'find-file
-                                    'dired-x-find-file
-                                    (current-global-map))
-         (substitute-key-definition 'find-file-other-window
-                                    'dired-x-find-file-other-window
-                                    (current-global-map)))
-        (t
-         (substitute-key-definition 'dired-x-find-file
-                                    'find-file
-                                    (current-global-map))
-         (substitute-key-definition 'dired-x-find-file-other-window
-                                    'find-file-other-window
-                                    (current-global-map))))
-  ;; Clear mini-buffer.
-  (message nil))
+  (interactive "P")
+  (let* ((guess (dired-filename-at-point))
+         (file (read-file-name "Find file: " guess guess nil nil)))
+    (if other-window
+        (find-file-other-window (expand-file-name file))
+      (find-file (expand-file-name file)))))
 
-;;; Now call it so binding is correct and put on `after-init-hook' in case
-;;; user changes binding.
-(dired-x-bind-find-file)
-(add-hook 'after-init-hook 'dired-x-bind-find-file)
+(fset 'find-this-file 'dired-find-this-file)
 
-(defun dired-x-find-file (filename)
-  "Edit file FILENAME.
-May create a new window, or reuse an existing one.
-See the function `display-buffer'.
-
-Identical to `find-file' except when called interactively, with a prefix arg
-\(e.g., \\[universal-argument]\), in which case it guesses filename near
-point.  Useful for editing file mentioned in buffer you are viewing, or to
-test if that file exists.  Use minibuffer after snatching filename."
-  (interactive (list (read-filename-at-point "Find file: ")))
-  (find-file (expand-file-name filename)))
-
-(defun dired-x-find-file-other-window (filename)
-  "Edit file FILENAME, in another window.
-May create a new window, or reuse an existing one.
-See the function `display-buffer'.
-
-Identical to `find-file-other-window' except when called interactively, with a
-prefix arg \(e.g., \\[universal-argument]\), in which case it guesses filename
-near point.  Useful for editing file mentioned in buffer you are viewing, or
-to test if that file exists.  Use minibuffer after snatching filename."
-  (interactive (list (read-filename-at-point "Find file: ")))
-  (find-file-other-window (expand-file-name filename)))
-
-;;; Internal functions.
+;;; Internal function.
 (defun dired-filename-at-point ()
 
   ;; Get the filename closest to point, but do not change position.  Has a
@@ -1598,24 +1590,13 @@ to test if that file exists.  Use minibuffer after snatching filename."
       ;; Return string.
       (expand-file-name (buffer-substring start (point))))))
 
-(defun read-filename-at-point (prompt)
-  ;;; Returns filename prompting with PROMPT with completion.  If
-  ;;; `current-prefix-arg' is non-nil, uses name at point as guess.
-  (if current-prefix-arg
-      (let ((guess (dired-filename-at-point)))
-        (read-file-name prompt
-                        (file-name-directory guess)
-                        guess
-                        nil (file-name-nondirectory guess)))
-    (read-file-name prompt default-directory)))
-
 
-;;; BUG REPORTS
+;;;; BUG REPORTS
 
 ;;; This section is provided for reports.  It uses Barry A. Warsaw's
 ;;; reporter.el which is bundled with GNU Emacs v19.
 
-(defconst dired-x-version "2.37"
+(defconst dired-x-version "2.27"
   "Revision number of dired-x.el -- dired extra for GNU Emacs v19.
 Type \\[dired-x-submit-report] to send a bug report.  Available via anonymous
 ftp in
