@@ -30,7 +30,7 @@
  *				pointer is left untouched and XM_FAILURE
  *				is returned.  
  *
- *			2)	When a selection request is recieved (i.e.,
+ *			2)	When a selection request is received (i.e.,
  *				when the specified mouse event occurs) the
  *				data pointer will be set to the data
  *				associated with the particular selection
@@ -354,6 +354,20 @@ XMenuActivate(display, menu, p_num, s_num, x_pos, y_pos, event_mask, data)
 		    _XMRefreshPane(display, menu, cur_p);
 		}
 		if (event_xmp->active) event_xmp->activated = True;
+#if 1
+		/*
+		 * i suspect the we don't get an EXPOSE event when backing
+		 * store is enabled; the menu windows content is probably
+		 * not drawn in when it should be in that case.
+		 * in that case, this is probably an ugly fix!
+		 * i hope someone more familiar with this code would
+		 * take it from here.  -- caveh@eng.sun.com.
+		 */
+		XSetWindowBackground(display,
+				     event_xmp->window, 
+				     menu->bkgnd_color);
+		_XMRefreshPane(display, menu, event_xmp);
+#endif
 		cur_p = event_xmp;
 	    }
 	    break;
@@ -381,7 +395,7 @@ XMenuActivate(display, menu, p_num, s_num, x_pos, y_pos, event_mask, data)
     case ButtonRelease:
 		*p_num = cur_p->serial;
 		/*
-		 * Check to see if there is a current selecion.
+		 * Check to see if there is a current selection.
 		 */
 		if (cur_s != NULL) {
 		    /*
