@@ -1,6 +1,6 @@
 /* Lisp parsing and input streams.
-   Copyright (C) 1985, 1986, 1987, 1988, 1989,
-   1993 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1988, 1989, 
+   1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -24,12 +24,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <ctype.h>
-#include "config.h"
+#include <config.h>
 #include "lisp.h"
 
 #ifndef standalone
 #include "buffer.h"
-#include "paths.h"
+#include <paths.h>
 #include "commands.h"
 #include "keyboard.h"
 #include "termhooks.h"
@@ -47,6 +47,14 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #endif
+
+#ifdef MSDOS
+/* These are redefined (correctly, but differently) in values.h.  */
+#undef INTBITS
+#undef LONGBITS
+#undef SHORTBITS
+#endif
+
 #include <math.h>
 #endif /* LISP_FLOAT_TYPE */
 
@@ -69,6 +77,9 @@ Lisp_Object Vload_history;
 /* This is useud to build the load history. */
 Lisp_Object Vcurrent_load_list;
 
+/* List of descriptors now open for Fload.  */
+static Lisp_Object load_descriptor_list;
+
 /* File for get_file_char to read from.  Use by load */
 static FILE *instream;
 
@@ -151,7 +162,11 @@ unreadchar (readcharfun, c)
      Lisp_Object readcharfun;
      int c;
 {
-  if (XTYPE (readcharfun) == Lisp_Buffer)
+  if (c == -1)
+    /* Don't back up the pointer if we're unreading the end-of-input mark,
+       since readchar didn't advance it when we read it.  */
+    ;
+  else if (XTYPE (readcharfun) == Lisp_Buffer)
     {
       if (XBUFFER (readcharfun) == current_buffer)
 	SET_PT (point - 1);
@@ -195,13 +210,17 @@ read_filtered_event (no_switch_frame, ascii_required, error_nonascii)
 #ifdef standalone
   return make_number (getchar ());
 #else
-  register Lisp_Object val;
-  register Lisp_Object delayed_switch_frame = Qnil;
+  register Lisp_Object val, delayed_switch_frame;
+
+  delayed_switch_frame = Qnil;
 
   /* Read until we get an acceptable event.  */
  retry:
   val = read_char (0, 0, 0, Qnil, 0);
 
+  if (XTYPE (val) == Lisp_Buffer)
+    goto retry;
+
   /* switch-frame events are put off until after the next ASCII
      character.  This is better than signalling an error just because
      the last characters were typed to a separate minibuffer frame,
@@ -237,7 +256,7 @@ read_filtered_event (no_switch_frame, ascii_required, error_nonascii)
 	{
 	  if (error_nonascii)
 	    {
-	      unread_command_events = Fcons (val, Qnil);
+	      Vunread_command_events = Fcons (val, Qnil);
 	      error ("Non-character input-event");
 	    }
 	  else
@@ -292,6 +311,7 @@ DEFUN ("get-file-char", Fget_file_char, Sget_file_char, 0, 0, 0,
 
 static void readevalloop ();
 static Lisp_Object load_unwind ();
+static Lisp_Object load_descriptor_unwind ();
 
 DEFUN ("load", Fload, Sload, 1, 4, 0,
   "Execute a file of Lisp code named FILE.\n\
@@ -319,12 +339,15 @@ Return t if file exists.")
   /* 1 means inhibit the message at the beginning.  */
   int nomessage1 = 0;
   Lisp_Object handler;
+#ifdef MSDOS
+  char *dosmode = "rt";
+#endif
 
   CHECK_STRING (str, 0);
   str = Fsubstitute_in_file_name (str);
 
   /* If file name is magic, call the handler.  */
-  handler = Ffind_file_name_handler (str);
+  handler = Ffind_file_name_handler (str, Qload);
   if (!NILP (handler))
     return call5 (handler, Qload, str, noerror, nomessage, nosuffix);
 
@@ -332,8 +355,10 @@ Return t if file exists.")
      since it would try to load a directory as a Lisp file */
   if (XSTRING (str)->size > 0)
     {
+      GCPRO1 (str);
       fd = openp (Vload_path, str, !NILP (nosuffix) ? "" : ".elc:.el:",
 		  &found, 0);
+      UNGCPRO;
     }
 
   if (fd < 0)
@@ -352,9 +377,12 @@ Return t if file exists.")
       struct stat s1, s2;
       int result;
 
-      stat (XSTRING (found)->data, &s1);
+#ifdef MSDOS
+      dosmode = "rb";
+#endif
+      stat ((char *)XSTRING (found)->data, &s1);
       XSTRING (found)->data[XSTRING (found)->size - 1] = 0;
-      result = stat (XSTRING (found)->data, &s2);
+      result = stat ((char *)XSTRING (found)->data, &s2);
       if (result >= 0 && (unsigned) s1.st_mtime < (unsigned) s2.st_mtime)
 	{
 	  message ("Source file `%s' newer than byte-compiled file",
@@ -366,7 +394,12 @@ Return t if file exists.")
       XSTRING (found)->data[XSTRING (found)->size - 1] = 'c';
     }
 
+#ifdef MSDOS
+  close (fd);
+  stream = fopen ((char *) XSTRING (found)->data, dosmode);
+#else
   stream = fdopen (fd, "r");
+#endif
   if (stream == 0)
     {
       close (fd);
@@ -384,6 +417,9 @@ Return t if file exists.")
   *ptr = stream;
   XSET (lispstream, Lisp_Internal_Stream, (int) ptr);
   record_unwind_protect (load_unwind, lispstream);
+  record_unwind_protect (load_descriptor_unwind, load_descriptor_list);
+  load_descriptor_list
+    = Fcons (make_number (fileno (stream)), load_descriptor_list);
   load_in_progress++;
   readevalloop (Qget_file_char, stream, str, Feval, 0);
   unbind_to (count, Qnil);
@@ -409,6 +445,23 @@ load_unwind (stream)  /* used as unwind-protect function in load */
   return Qnil;
 }
 
+static Lisp_Object
+load_descriptor_unwind (oldlist)
+     Lisp_Object oldlist;
+{
+  load_descriptor_list = oldlist;
+}
+
+/* Close all descriptors in use for Floads.
+   This is used when starting a subprocess.  */
+
+void
+close_load_descs ()
+{
+  Lisp_Object tail;
+  for (tail = load_descriptor_list; !NILP (tail); tail = XCONS (tail)->cdr)
+    close (XFASTINT (XCONS (tail)->car));
+}
 
 static int
 complete_filename_p (pathname)
@@ -422,6 +475,9 @@ complete_filename_p (pathname)
 #ifdef VMS
 	  || index (s, ':')
 #endif /* VMS */
+#ifdef MSDOS	/* MW, May 1993 */
+	  || (s[0] != '\0' && s[1] == ':' && s[2] == '/')
+#endif
 	  );
 }
 
@@ -453,7 +509,9 @@ openp (path, str, suffix, storeptr, exec_only)
   int want_size;
   register Lisp_Object filename;
   struct stat st;
+  struct gcpro gcpro1;
 
+  GCPRO1 (str);
   if (storeptr)
     *storeptr = Qnil;
 
@@ -504,14 +562,14 @@ openp (path, str, suffix, storeptr, exec_only)
 	      if (exec_only)
 		fd = (access (fn, X_OK) == 0) ? 1 : -1;
 	      else
-		fd = open (fn, 0, 0);
+		fd = open (fn, O_RDONLY, 0);
 
 	      if (fd >= 0)
 		{
 		  /* We succeeded; return this descriptor and filename.  */
 		  if (storeptr)
 		    *storeptr = build_string (fn);
-		  return fd;
+		  RETURN_UNGCPRO (fd);
 		}
 	    }
 
@@ -520,10 +578,11 @@ openp (path, str, suffix, storeptr, exec_only)
 	    break;
 	  nsuffix += lsuffix + 1;
 	}
-      if (absolute) return -1;
+      if (absolute)
+	RETURN_UNGCPRO (-1);
     }
 
-  return -1;
+  RETURN_UNGCPRO (-1);
 }
 
 
@@ -618,6 +677,12 @@ readevalloop (readcharfun, stream, sourcename, evalfun, printflag)
   register Lisp_Object val;
   int count = specpdl_ptr - specpdl;
   struct gcpro gcpro1;
+  struct buffer *b = 0;
+
+  if (BUFFERP (readcharfun))
+    b = XBUFFER (readcharfun);
+  else if (MARKERP (readcharfun))
+    b = XMARKER (readcharfun)->buffer;
 
   specbind (Qstandard_input, readcharfun);
   specbind (Qcurrent_load_list, Qnil);
@@ -628,6 +693,9 @@ readevalloop (readcharfun, stream, sourcename, evalfun, printflag)
 
   while (1)
     {
+      if (b != 0 && NILP (b->name))
+	error ("Reading from killed buffer");
+
       instream = stream;
       c = READCHAR;
       if (c == ';')
@@ -1144,11 +1212,21 @@ read1 (readcharfun)
 		if (p == read_buffer)
 		  cancel = 1;
 	      }
-	    else if (c & CHAR_META)
-	      /* Move the meta bit to the right place for a string.  */
-	      *p++ = (c & ~CHAR_META) | 0x80;
 	    else
-	      *p++ = c;
+	      {
+		/* Allow `\C- ' and `\C-?'.  */
+		if (c == (CHAR_CTL | ' '))
+		  c = 0;
+		else if (c == (CHAR_CTL | '?'))
+		  c = 127;
+
+		if (c & CHAR_META)
+		  /* Move the meta bit to the right place for a string.  */
+		  c = (c & ~CHAR_META) | 0x80;
+		if (c & ~0xff)
+		  error ("Invalid modifier in string");
+		*p++ = c;
+	      }
 	  }
 	if (c < 0) return Fsignal (Qend_of_file, Qnil);
 
@@ -1189,6 +1267,7 @@ read1 (readcharfun)
       if (c <= 040) goto retry;
       {
 	register char *p = read_buffer;
+	int quoted = 0;
 
 	{
 	  register char *end = read_buffer + read_buffer_size;
@@ -1212,7 +1291,10 @@ read1 (readcharfun)
 		  end = read_buffer + read_buffer_size;
 		}
 	      if (c == '\\')
-		c = READCHAR;
+		{
+		  c = READCHAR;
+		  quoted = 1;
+		}
 	      *p++ = c;
 	      c = READCHAR;
 	    }
@@ -1229,35 +1311,36 @@ read1 (readcharfun)
 	    UNREAD (c);
 	}
 
-	/* Is it an integer? */
-	{
-	  register char *p1;
-	  register Lisp_Object val;
-	  p1 = read_buffer;
-	  if (*p1 == '+' || *p1 == '-') p1++;
-	  if (p1 != p)
-	    {
-	      while (p1 != p && (c = *p1) >= '0' && c <= '9') p1++;
+	if (!quoted)
+	  {
+	    register char *p1;
+	    register Lisp_Object val;
+	    p1 = read_buffer;
+	    if (*p1 == '+' || *p1 == '-') p1++;
+	    /* Is it an integer? */
+	    if (p1 != p)
+	      {
+		while (p1 != p && (c = *p1) >= '0' && c <= '9') p1++;
 #ifdef LISP_FLOAT_TYPE
-	      /* Integers can have trailing decimal points.  */
-	      if (p1 > read_buffer && p1 < p && *p1 == '.') p1++;
+		/* Integers can have trailing decimal points.  */
+		if (p1 > read_buffer && p1 < p && *p1 == '.') p1++;
 #endif
-	      if (p1 == p)
-		/* It is an integer. */
-		{
+		if (p1 == p)
+		  /* It is an integer. */
+		  {
 #ifdef LISP_FLOAT_TYPE
-		  if (p1[-1] == '.')
-		    p1[-1] = '\0';
+		    if (p1[-1] == '.')
+		      p1[-1] = '\0';
 #endif
-		  XSET (val, Lisp_Int, atoi (read_buffer));
-		  return val;
-		}
-	    }
+		    XSET (val, Lisp_Int, atoi (read_buffer));
+		    return val;
+		  }
+	      }
 #ifdef LISP_FLOAT_TYPE
-	  if (isfloat_string (read_buffer))
-	    return make_float (atof (read_buffer));
+	    if (isfloat_string (read_buffer))
+	      return make_float (atof (read_buffer));
 #endif
-	}
+	  }
 
 	return intern (read_buffer);
       }
@@ -1441,8 +1524,9 @@ intern (str)
 {
   Lisp_Object tem;
   int len = strlen (str);
-  Lisp_Object obarray = Vobarray;
+  Lisp_Object obarray;
 
+  obarray = Vobarray;
   if (XTYPE (obarray) != Lisp_Vector || XVECTOR (obarray)->size == 0)
     obarray = check_obarray (obarray);
   tem = oblookup (obarray, str, len);
@@ -1516,8 +1600,8 @@ oblookup (obarray, ptr, size)
   register Lisp_Object tail;
   Lisp_Object bucket, tem;
 
-  if (XTYPE (obarray) != Lisp_Vector ||
-      (obsize = XVECTOR (obarray)->size) == 0)
+  if (XTYPE (obarray) != Lisp_Vector
+      || (obsize = XVECTOR (obarray)->size) == 0)
     {
       obarray = check_obarray (obarray);
       obsize = XVECTOR (obarray)->size;
@@ -1605,7 +1689,7 @@ OBARRAY defaults to the value of `obarray'.")
   return Qnil;
 }
 
-#define OBARRAY_SIZE 509
+#define OBARRAY_SIZE 1511
 
 void
 init_obarray ()
@@ -1784,7 +1868,27 @@ init_lread ()
 
       dump_path = decode_env_path (0, PATH_DUMPLOADSEARCH);
       if (! NILP (Fequal (dump_path, Vload_path)))
-	Vload_path = decode_env_path (0, normal);
+	{
+	  Vload_path = decode_env_path (0, normal);
+	  if (!NILP (Vinstallation_directory))
+	    {
+	      /* Add to the path the lisp subdir of the
+		 installation dir, if it exists.  */
+	      Lisp_Object tem, tem1;
+	      tem = Fexpand_file_name (build_string ("lisp"),
+				       Vinstallation_directory);
+	      tem1 = Ffile_exists_p (tem);
+	      if (!NILP (tem1))
+		{
+		  if (NILP (Fmember (tem, Vload_path)))
+		    Vload_path = nconc2 (Vload_path, Fcons (tem, Qnil));
+		}
+	      else
+		/* That dir doesn't exist, so add the build-time
+		   Lisp dirs instead.  */
+		Vload_path = nconc2 (Vload_path, dump_path);
+	    }
+	}
     }
   else
     Vload_path = decode_env_path (0, normal);
@@ -1804,8 +1908,9 @@ init_lread ()
 	  {
 	    dirfile = Fdirectory_file_name (dirfile);
 	    if (access (XSTRING (dirfile)->data, 0) < 0)
-	      printf ("Warning: lisp library (%s) does not exist.\n",
-		      XSTRING (Fcar (path_tail))->data);
+	      fprintf (stderr,
+		       "Warning: Lisp directory `%s' does not exist.\n",
+		       XSTRING (Fcar (path_tail))->data);
 	  }
       }
   }
@@ -1819,6 +1924,8 @@ init_lread ()
   Vvalues = Qnil;
 
   load_in_progress = 0;
+
+  load_descriptor_list = Qnil;
 }
 
 void
@@ -1885,6 +1992,9 @@ or variables, and cons cells `(provide . FEATURE)' and `(require . FEATURE)'.");
     "Used for internal purposes by `load'.");
   Vcurrent_load_list = Qnil;
 
+  load_descriptor_list = Qnil;
+  staticpro (&load_descriptor_list);
+
   Qcurrent_load_list = intern ("current-load-list");
   staticpro (&Qcurrent_load_list);
 
