@@ -1,5 +1,5 @@
 /* Work-alike for termcap, plus extra features.
-   Copyright (C) 1985, 86, 93, 94, 95 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1993, 1994 Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,24 +13,17 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Emacs config.h may rename various library functions such as malloc.  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
+#else /* not HAVE_CONFIG_H */
+
+#if defined(HAVE_STRING_H) || defined(STDC_HEADERS)
+#define bcopy(s, d, n) memcpy ((d), (s), (n))
 #endif
 
-#ifdef emacs
-
-/* Get the O_* definitions for open et al.  */
-#include <sys/file.h>
-#ifdef USG5
-#include <fcntl.h>
-#endif
-
-#else /* not emacs */
-
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -40,11 +33,6 @@ char *malloc ();
 char *realloc ();
 #endif
 
-/* Do this after the include, in case string.h prototypes bcopy.  */
-#if (defined(HAVE_STRING_H) || defined(STDC_HEADERS)) && !defined(bcopy)
-#define bcopy(s, d, n) memcpy ((d), (s), (n))
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -52,16 +40,12 @@ char *realloc ();
 #include <fcntl.h>
 #endif
 
-#endif /* not emacs */
+#endif /* not HAVE_CONFIG_H */
 
 #ifndef NULL
 #define NULL (char *) 0
 #endif
 
-#ifndef O_RDONLY
-#define O_RDONLY 0
-#endif
-
 /* BUFSIZE is the initial size allocated for the buffer
    for reading the termcap file.
    It is not a limit.
@@ -79,8 +63,8 @@ int bufsize = 128;
 #endif
 #endif
 
-#ifndef TERMCAP_FILE
-#define TERMCAP_FILE "/etc/termcap"
+#ifndef TERMCAP_NAME
+#define TERMCAP_NAME "/etc/termcap"
 #endif
 
 #ifndef emacs
@@ -223,13 +207,7 @@ tgetst1 (ptr, area)
   while ((c = *p++) && c != ':' && c != '\n')
     {
       if (c == '^')
-	{
-	  c = *p++;
-	  if (c == '?')
-	    c = 0177;
-	  else
-	    c &= 037;
-	}
+	c = *p++ & 037;
       else if (c == '\\')
 	{
 	  c = *p++;
@@ -271,14 +249,14 @@ char PC;
 /* Actual baud rate if positive;
    - baud rate / 100 if negative.  */
 
-static int speeds[] =
+static short speeds[] =
   {
 #ifdef VMS
     0, 50, 75, 110, 134, 150, -3, -6, -12, -18,
     -20, -24, -36, -48, -72, -96, -192
 #else /* not VMS */
     0, 50, 75, 110, 135, 150, -2, -3, -6, -12,
-    -18, -24, -48, -96, -192, -288, -384, -576, -1152
+    -18, -24, -48, -96, -192, -384
 #endif /* not VMS */
   };
 
@@ -294,17 +272,11 @@ tputs (str, nlines, outfun)
 #ifdef emacs
   extern baud_rate;
   speed = baud_rate;
-  /* For quite high speeds, convert to the smaller
-     units to avoid overflow.  */
-  if (speed > 10000)
-    speed = - speed / 100;
 #else
   if (ospeed == 0)
     speed = tputs_baud_rate;
-  else if (ospeed > 0 && ospeed < (sizeof speeds / sizeof speeds[0]))
-    speed = speeds[ospeed];
   else
-    speed = 0;
+    speed = speeds[ospeed];
 #endif
 
   if (!str)
@@ -328,14 +300,11 @@ tputs (str, nlines, outfun)
   while (*str)
     (*outfun) (*str++);
 
-  /* PADCOUNT is now in units of tenths of msec.
-     SPEED is measured in characters per 10 seconds
-     or in characters per .1 seconds (if negative).
-     We use the smaller units for larger speeds to avoid overflow.  */
-  padcount *= speed;
+  /* padcount is now in units of tenths of msec.  */
+  padcount *= speeds[ospeed];
   padcount += 500;
   padcount /= 1000;
-  if (speed < 0)
+  if (speeds[ospeed] < 0)
     padcount = -padcount;
   else
     {
@@ -424,7 +393,7 @@ tgetent (bp, name)
   register int fd;
   struct buffer buf;
   register char *bp1;
-  char *tc_search_point;
+  char *bp2;
   char *term;
   int malloc_size = 0;
   register int c;
@@ -448,11 +417,6 @@ tgetent (bp, name)
     }
 #endif /* INTERNAL_TERMINAL */
 
-  /* For compatibility with programs like `less' that want to
-     put data in the termcap buffer themselves as a fallback.  */
-  if (bp)
-    term_entry = bp;
-
   termcap_name = getenv ("TERMCAP");
   if (termcap_name && *termcap_name == '\0')
     termcap_name = NULL;
@@ -490,14 +454,14 @@ tgetent (bp, name)
     }
 
   if (!termcap_name || !filep)
-    termcap_name = TERMCAP_FILE;
+    termcap_name = TERMCAP_NAME;
 
   /* Here we know we must search a file and termcap_name has its name.  */
 
 #ifdef MSDOS
-  fd = open (termcap_name, O_RDONLY|O_TEXT, 0);
+  fd = open (termcap_name, O_TEXT, 0);
 #else
-  fd = open (termcap_name, O_RDONLY, 0);
+  fd = open (termcap_name, 0, 0);
 #endif
   if (fd < 0)
     return -1;
@@ -512,7 +476,7 @@ tgetent (bp, name)
       malloc_size = indirect ? strlen (tcenv) + 1 : buf.size;
       bp = (char *) xmalloc (malloc_size);
     }
-  tc_search_point = bp1 = bp;
+  bp1 = bp;
 
   if (indirect)
     /* Copy the data from the environment variable.  */
@@ -543,10 +507,11 @@ tgetent (bp, name)
 	  malloc_size = bp1 - bp + buf.size;
 	  termcap_name = (char *) xrealloc (bp, malloc_size);
 	  bp1 += termcap_name - bp;
-	  tc_search_point += termcap_name - bp;
 	  bp = termcap_name;
 	}
 
+      bp2 = bp1;
+
       /* Copy the line of the entry from buf into bp.  */
       termcap_name = buf.ptr;
       while ((*bp1++ = c = *termcap_name++) && c != '\n')
@@ -560,8 +525,7 @@ tgetent (bp, name)
 
       /* Does this entry refer to another terminal type's entry?
 	 If something is found, copy it into heap and null-terminate it.  */
-      tc_search_point = find_capability (tc_search_point, "tc");
-      term = tgetst1 (tc_search_point, (char **) 0);
+      term = tgetst1 (find_capability (bp2, "tc"), (char **) 0);
     }
 
   close (fd);
@@ -572,6 +536,8 @@ tgetent (bp, name)
 
  ret:
   term_entry = bp;
+  if (malloc_size)
+    return (int) bp;
   return 1;
 }
 
@@ -782,3 +748,4 @@ tprint (cap)
 }
 
 #endif /* TEST */
+
